@@ -35,13 +35,16 @@ from disent.dataset.sampling._base import BaseDisentSampler
 
 class RandomSampler(BaseDisentSampler):
 
+    def uninit_copy(self) -> 'RandomSampler':
+        return RandomSampler(num_samples=self.num_samples)
+
     def __init__(self, num_samples=1):
         super().__init__(num_samples=num_samples)
 
     def _init(self, dataset):
         self._len = len(dataset)
 
-    def __call__(self, idx: int) -> Tuple[int, ...]:
+    def _sample_idx(self, idx: int) -> Tuple[int, ...]:
         # sample indices
         return (idx, *np.random.randint(0, self._len, size=self._num_samples-1))
 
