@@ -34,8 +34,6 @@ import wandb
 from omegaconf import DictConfig
 from omegaconf import ListConfig
 from omegaconf import OmegaConf
-from pytorch_lightning.loggers import CometLogger
-from pytorch_lightning.loggers import LoggerCollection
 from pytorch_lightning.loggers import WandbLogger
 
 from disent import metrics
@@ -45,12 +43,12 @@ from disent.model import AutoEncoder
 from disent.nn.weights import init_model_weights
 from disent.util.seeds import seed
 from disent.util.strings.fmt import make_box_str
+from disent.util.strings import colors as c
 from disent.util.lightning.callbacks import LoggerProgressCallback
 from disent.util.lightning.callbacks import VaeMetricLoggingCallback
 from disent.util.lightning.callbacks import VaeLatentCycleLoggingCallback
 from disent.util.lightning.callbacks import VaeGtDistsLoggingCallback
 from experiment.util.hydra_data import HydraDataModule
-from experiment.util.hydra_utils import make_non_strict
 from experiment.util.run_utils import log_error_and_exit
 from experiment.util.run_utils import safe_unset_debug_logger
 from experiment.util.run_utils import safe_unset_debug_trainer
@@ -66,23 +64,24 @@ log = logging.getLogger(__name__)
 # ========================================================================= #
 
 
-def hydra_check_cuda(cfg):
-    cuda = cfg.dsettings.trainer.cuda
-    # set cuda
-    if cuda in {'try_cuda', None}:
-        cfg.dsettings.trainer.cuda = torch.cuda.is_available()
-        if not cuda:
+def hydra_get_gpus(cfg) -> int:
+    use_cuda = cfg.dsettings.trainer.cuda
+    # check cuda values
+    if use_cuda in {'try_cuda', None}:
+        use_cuda = torch.cuda.is_available()
+        if not use_cuda:
             log.warning('CUDA was requested, but not found on this system... CUDA has been disabled!')
+    elif use_cuda:
+        if not torch.cuda.is_available():
+            log.error('`dsettings.trainer.cuda=True` but CUDA is not available on this machine!')
+            raise RuntimeError('CUDA not available!')
     else:
         if not torch.cuda.is_available():
-            if cuda:
-                log.error('trainer.cuda=True but CUDA is not available on this machine!')
-                raise RuntimeError('CUDA not available!')
-            else:
-                log.warning('CUDA is not available on this machine!')
+            log.info('CUDA is not available on this machine!')
         else:
-            if not cuda:
-                log.warning('CUDA is available but is not being used!')
+            log.warning('CUDA is available but is not being used!')
+    # get number of gpus to use
+    return (1 if use_cuda else 0)
 
 
 def hydra_check_data_paths(cfg):
@@ -244,11 +243,9 @@ def hydra_create_and_update_framework_config(cfg) -> DisentConfigurable.cfg:
     # warn if some of the cfg variables were not overridden
     missing_keys = sorted(set(framework_cfg.get_keys()) - (set(cfg.framework.cfg.keys())))
     if missing_keys:
-        log.error(f'Framework {repr(cfg.framework.name)} is missing config keys for:')
+        log.warning(f'{c.RED}Framework {repr(cfg.framework.name)} is missing config keys for:{c.RST}')
         for k in missing_keys:
-            log.error(f'{repr(k)}')
-    # update config params in case we missed variables in the cfg
-    cfg.framework.cfg.update(framework_cfg.to_dict())
+            log.warning(f'{c.RED}{repr(k)}{c.RST}')
     # return config
     return framework_cfg
 
@@ -277,36 +274,29 @@ def hydra_create_framework(framework_cfg: DisentConfigurable.cfg, datamodule, cf
 
 
 # ========================================================================= #
-# ACTIONS                                                                    #
+# ACTIONS                                                                   #
 # ========================================================================= #
 
 
-def prepare_data(cfg: DictConfig, config_path: str = None):
+def action_prepare_data(cfg: DictConfig):
     # get the time the run started
     time_string = datetime.today().strftime('%Y-%m-%d--%H-%M-%S')
     log.info(f'Starting run at time: {time_string}')
-    raise NotImplementedError
-
-    # # allow the cfg to be edited
-    # cfg = make_non_strict(cfg)
-    # # deterministic seed
-    # seed(cfg.job.setdefault('seed', None))
-    # # print useful info
-    # log.info(f"Current working directory : {os.getcwd()}")
-    # log.info(f"Orig working directory    : {hydra.utils.get_original_cwd()}")
-    # # hydra config does not support variables in defaults lists, we handle this manually
-    # cfg = merge_specializations(cfg, config_path=CONFIG_PATH if (config_path is None) else config_path, required=['_dataset_sampler_'])
-    # # check data preparation
-    # prepare_data_per_node = cfg.trainer.setdefault('prepare_data_per_node', True)
-    # hydra_check_datadir(prepare_data_per_node, cfg)
-    # # print the config
-    # log.info(f'Dataset Config Is:\n{make_box_str(OmegaConf.to_yaml({"dataset": cfg.dataset}))}')
-    # # prepare data
-    # datamodule = HydraDataModule(cfg)
-    # datamodule.prepare_data()
+    # deterministic seed
+    seed(cfg.settings.job.seed)
+    # print useful info
+    log.info(f"Current working directory : {os.getcwd()}")
+    log.info(f"Orig working directory    : {hydra.utils.get_original_cwd()}")
+    # check data preparation
+    hydra_check_data_paths(cfg)
+    # print the config
+    log.info(f'Dataset Config Is:\n{make_box_str(OmegaConf.to_yaml({"dataset": cfg.dataset}))}')
+    # prepare data
+    datamodule = HydraDataModule(cfg)
+    datamodule.prepare_data()
 
 
-def train(cfg: DictConfig, config_path: str = None):
+def action_train(cfg: DictConfig):
 
     # get the time the run started
     time_string = datetime.today().strftime('%Y-%m-%d--%H-%M-%S')
@@ -324,9 +314,6 @@ def train(cfg: DictConfig, config_path: str = None):
 
     # -~-~-~-~-~-~-~-~-~-~-~-~- #
 
-    # allow the cfg to be edited
-    cfg = make_non_strict(cfg)
-
     # deterministic seed
     seed(cfg.settings.job.seed)
 
@@ -342,7 +329,7 @@ def train(cfg: DictConfig, config_path: str = None):
     log.info(f"Orig working directory    : {hydra.utils.get_original_cwd()}")
 
     # check CUDA setting
-    hydra_check_cuda(cfg)
+    gpus = hydra_get_gpus(cfg)
 
     # check data preparation
     hydra_check_data_paths(cfg)
@@ -365,7 +352,7 @@ def train(cfg: DictConfig, config_path: str = None):
     trainer = set_debug_trainer(pl.Trainer(
         logger=logger,
         callbacks=callbacks,
-        gpus=1 if cfg.dsettings.trainer.cuda else 0,
+        gpus=gpus,
         # we do this here too so we don't run the final
         # metrics, even through we check for it manually.
         terminate_on_nan=True,
@@ -401,14 +388,35 @@ def train(cfg: DictConfig, config_path: str = None):
     #    initialising the training process we cannot capture it!
     trainer.fit(framework, datamodule=datamodule)
 
+    # -~-~-~-~-~-~-~-~-~-~-~-~- #
+
+    # cleanup this run
+    try:
+        wandb.finish()
+    except:
+        pass
+
+    # -~-~-~-~-~-~-~-~-~-~-~-~- #
+
 
 # available actions
 ACTIONS = {
-    'prepare_data': prepare_data,
-    'train': train,
+    'prepare_data': action_prepare_data,
+    'train': action_train,
+    'skip': lambda *args, **kwargs: None,
 }
 
 
+def run_action(cfg: DictConfig):
+    action_key = cfg.action
+    # get the action
+    if action_key not in ACTIONS:
+        raise KeyError(f'The given action: {repr(action_key)} is invalid, must be one of: {sorted(ACTIONS.keys())}')
+    action = ACTIONS[action_key]
+    # run the action
+    action(cfg)
+
+
 # ========================================================================= #
 # MAIN                                                                      #
 # ========================================================================= #
@@ -436,13 +444,7 @@ if __name__ == '__main__':
     @hydra.main(config_path=CONFIG_PATH, config_name=CONFIG_NAME)
     def hydra_main(cfg: DictConfig):
         try:
-            action_key = cfg.action
-            # get the action
-            if action_key not in ACTIONS:
-                raise KeyError(f'The given action: {repr(action_key)} is invalid, must be one of: {sorted(ACTIONS.keys())}')
-            action = ACTIONS[action_key]
-            # run the action
-            action(cfg)
+            run_action(cfg)
         except Exception as e:
             log_error_and_exit(err_type='experiment error', err_msg=str(e), exc_info=True)
         except:
